#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Aug 21 11:47:33 2024

@author: pteysseyre
"""

#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Aug 21 09:59:31 2024

@author: pteysseyre
"""

import numpy as np
from pymatreader import read_mat # To read the mau files
import os.path # Useful to determine if a file exist or not (to write headers in a new file)
import glob

#======================== Read mau files ======================================

def read_Narrowband_postprocessing(la_date, la_station, path): # For the moment, only takes the calibration after 2023_05_16
    """
    Reads the data for the station and the day asked and stores them in the output arrays
    Needs the path to the data files
    Reads the .mat files (already ended)
    Written by C. Briand on 16//10/2022
    Converted to Python on 18/03/2024"""

    
    # Copy the files read to another folder
    # pathdest = 'C:/Users/VLF4IONS/Desktop/VLF/NarrowbandSync/copies/'
    
    the_year, the_month, the_day = la_date.split('_')
    
    Cal_NS = 0.14
    Cal_EW = 0.08
    
    nb_errors = 0
    
    # amp_NS
    list_of_files = glob.glob(path+'NC*'+la_station+'_100A.mat')
    latest_file = max(list_of_files, key=os.path.getctime)
    #fname = shutil.copy2(f, pathdest) # If we want to copy the files, decomment the line and change the following f to fname
    try :
        data = read_mat(latest_file)
        amp_NS = data['data']*Cal_NS
        start_time = data['start_hour'] + data['start_minute']/60 + data['start_second']/3600
    except (TypeError, ValueError):
        print('TypeError - amp_NS')
        amp_NS = 0
        nb_errors += 1
        start_time = 0
    #os.remove(fname)
    
    # phase_EW
    list_of_files = glob.glob(path+'NC*'+la_station+'_101B.mat')
    latest_file = max(list_of_files, key=os.path.getctime)
        #fname = shutil.copy2(f, pathdest)
    try :
        data = read_mat(latest_file)
        phase_EW = data['data']
        start_time = data['start_hour'] + data['start_minute']/60 + data['start_second']/3600
    except (TypeError, ValueError):
        print('TypeError - phase_EW')
        
        # If there is a pb with the phase_EW files, a solution is to take the phase_NS ones
        # phase_NS
        list_of_files = glob.glob(path+'NC*'+la_station+'_100B.mat')
        latest_file = max(list_of_files, key=os.path.getctime)
            #fname = shutil.copy2(f, pathdest)
        try :
            data = read_mat(latest_file)
            phase_EW = data['data']
            start_time = data['start_hour'] + data['start_minute']/60 + data['start_second']/3600
        except (TypeError, ValueError):
            print('TypeError - phase_NS')
            phase_EW = 0
            nb_errors += 1
            start_time = 0
            #os.remove(fname)
    
    
    # amp_EW
    list_of_files = glob.glob(path+'NC*'+la_station+'_101A.mat')
    latest_file = max(list_of_files, key=os.path.getctime)
        #fname = shutil.copy2(f, pathdest)
    try :
        data = read_mat(latest_file)
        amp_EW = data['data']*Cal_EW
        start_time = data['start_hour'] + data['start_minute']/60 + data['start_second']/3600
    except (TypeError, ValueError):
        print('TypeError - amp_EW')
        amp_EW = 0
        nb_errors += 1
        start_time = 0
        #os.remove(fname)

    # phase_NS
    list_of_files = glob.glob(path+'NC*'+la_station+'_100B.mat')
    latest_file = max(list_of_files, key=os.path.getctime)
        #fname = shutil.copy2(f, pathdest)
    try :
        data = read_mat(latest_file)
        phase_NS = data['data']
        start_time = data['start_hour'] + data['start_minute']/60 + data['start_second']/3600
    except (TypeError, ValueError):
        print('TypeError - phase_NS')
        phase_NS = 0
        nb_errors += 1
        start_time = 0
        #os.remove(fname)
    
    Fs = data['Fs']
    tstep = 1/Fs/3600
    the_time = np.linspace(start_time, tstep*np.size(phase_EW), np.size(phase_EW))
    
    return (amp_NS, amp_EW, phase_NS, phase_EW, the_time, nb_errors)

def read_Narrowband_real_time(la_date, la_station, path): # For the moment, only takes the calibration after 2023_05_16

    """
    Reads the data for the station and the day asked and stores them in the output arrays
    Reads the .mat files still being written
    Needs the path to the files
    Written by C. Briand on 16//10/2022
    Converted to Python on 18/03/2024"""

    
    # Copy the files read to another folder
    # pathdest = 'C:/Users/VLF4IONS/Desktop/VLF/NarrowbandSync/copies/'
    
    the_year, the_month, the_day = la_date.split('_')
    
    Cal_NS = 0.14
    Cal_EW = 0.08

    nb_errors = 0
    
    # amp_NS
    list_of_files = glob.glob(path+'*'+la_station+'_100A.mau')
    latest_file = max(list_of_files, key=os.path.getctime)
    #fname = shutil.copy2(f, pathdest) # If we want to copy the files, decomment the line and change the following f to fname
    try :
        data = read_mat(latest_file)
        amp_NS = data['data']*Cal_NS
        start_time = data['start_hour'] + data['start_minute']/60 + data['start_second']/3600
    except (TypeError, ValueError):
        print('TypeError - amp_NS')
        amp_NS = 0
        nb_errors += 1
        start_time = 0
    #os.remove(fname)
    
    # phase_EW
    list_of_files = glob.glob(path+'*'+la_station+'_101B.mau')
    latest_file = max(list_of_files, key=os.path.getctime)
        #fname = shutil.copy2(f, pathdest)
    try :
        data = read_mat(latest_file)
        phase_EW = data['data']
        start_time = data['start_hour'] + data['start_minute']/60 + data['start_second']/3600
    except (TypeError, ValueError):
        print('TypeError - phase_EW')
        
        # If there is a pb with the phase_EW files, a solution is to take the phase_NS ones
        # phase_NS
        list_of_files = glob.glob(path+'*'+la_station+'_100B.mau')
        latest_file = max(list_of_files, key=os.path.getctime)
            #fname = shutil.copy2(f, pathdest)
        try :
            data = read_mat(latest_file)
            phase_EW = data['data']
            start_time = data['start_hour'] + data['start_minute']/60 + data['start_second']/3600
        except (TypeError, ValueError):
            print('TypeError - phase_NS')
            phase_EW = 0
            nb_errors += 1
            start_time = 0
            #os.remove(fname)
    
    
    # amp_EW
    list_of_files = glob.glob(path+'*'+la_station+'_101A.mau')
    latest_file = max(list_of_files, key=os.path.getctime)
        #fname = shutil.copy2(f, pathdest)
    try :
        data = read_mat(latest_file)
        amp_EW = data['data']*Cal_EW
        start_time = data['start_hour'] + data['start_minute']/60 + data['start_second']/3600
    except (TypeError, ValueError):
        print('TypeError - amp_EW')
        amp_EW = 0
        nb_errors += 1
        start_time = 0
        #os.remove(fname)

    # phase_NS
    list_of_files = glob.glob(path+'*'+la_station+'_100B.mau')
    latest_file = max(list_of_files, key=os.path.getctime)
        #fname = shutil.copy2(f, pathdest)
    try :
        data = read_mat(latest_file)
        phase_NS = data['data']
        start_time = data['start_hour'] + data['start_minute']/60 + data['start_second']/3600
    except (TypeError, ValueError):
        print('TypeError - phase_NS')
        phase_NS = 0
        nb_errors += 1
        start_time = 0
        #os.remove(fname)
    
    Fs = data['Fs']
    tstep = 1/Fs/3600
    the_time = np.linspace(start_time, tstep*np.size(phase_EW), np.size(phase_EW))
    
    return (amp_NS, amp_EW, phase_NS, phase_EW, the_time, nb_errors)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Aug 21 10:41:33 2024

@author: pteysseyre
"""

import numpy as np
import pandas as pd # NOTE : we only need this to read csv files; there may be an easier way to do this, though that works too

def LMP_findminimum(la_station, amp, goodphase, pathtoCSVfiles):
    """ Gives the time profile of H, B
    # NOTE : Make sure that the relevant csv files are stored on the computer AND that they have been detrended
    # They should also only contain A/P values (not H' and beta)
    # Lastly, because of the detrend, the modelled phase may be shifted by a multiple of 90° 
    # The convention chosen here is that the phase is between 90 and 180° at 74 km / 0.3 km^-1
    #---------------------
    # Inputs : 
    # - la_station / la_date : transmitter call sign and date of insterest (ex:
    # 'GBZ', '2023_09_21')
    # - amp, goodphase : measured amplitude and phase (in pT and °). The phase must have been corrected beforehand
    # --------------------------
    # Output : 
    # - H, B : H' and beta arrays
    
    # History : written in march 2024 by P.Teysseyre
    # Based on golden ratio method"""
        
    #the_year, the_month, the_day = la_date.split('_')
    #ladate = the_year + the_month + the_day
    
    data = np.array([amp, goodphase])
    
    # Read the files for modelled amplitude and phase
    # DO NOT open the files with Numbers before !
    A = pd.read_csv(pathtoCSVfiles+'Amplitude'+la_station+'.csv', header=None)
    P = pd.read_csv(pathtoCSVfiles+'Phase'+la_station+'.csv', header=None)
    
    A = A.to_numpy()
    P = P.to_numpy()

    # Get the ranges of parameters H' and beta
    h = np.arange(50, 87.1, 0.1)
    b = np.arange(0.1, 0.61, 0.01)
    
    #----------------------------
    # From that point onwards, the code compares the data and the modelled amp and phase to get H' and beta
    
        
    (rows, columns) = np.shape(A)
    
    # We remove all the NaNs
    P = P[~np.isnan(A)]
    A = A[~np.isnan(A)]
    
    
    # Compute the matrices of relative errors:
    Aq = np.abs(A - data[0])/(data[0] + 1e-2)
    Pq = np.abs(P - data[1])/(data[1] + 1e-2)
    
    # We normalise the matrices (otherwise the errors on the phase are given too much weight 
    # as the phase has a much higher range than the amplitude)
    A_new = (Aq - np.min(Aq))/(np.max(Aq) - np.min(Aq))
    P_new = (Pq - np.min(Pq))/(np.max(Pq) - np.min(Pq))
    
    # The aim is to find eps such that only one point has an error in phase and amp less than eps
    
    # For the golden ratio method, four points are necessary:
    #   - the upper point will be eps_up, with corresponding matrices A_new, P_new and rc
    #   - then eps_3, with A_new3, P_new3, rc3 and rc_new3
    #   - then eps_4, with A_new4, P_new4, rc4, rc_new4
    #   - lastly eps_down, with no corresponding matrix
    
    # A_new's and P_new's are the amplitude and phase of the points with relative 

    
    # Loop counter:
    i = 0
    
    # Golden ratio number
    r = 1/((1 + np.sqrt(5))/2)
    
    # Initialisation of the interations 
    eps_up = np.max([np.max(A_new), np.max(P_new)])
    eps_down = 0    
    interval = eps_up - eps_down
    eps_4 = r**2 * interval + eps_down
    eps_3 = r * interval + eps_down
    
    # Initialisation of rc (Technically, only the last line is absolutely necessary. The lines before just flatten A_new and P_new)
    rc = ((A_new <= eps_up) & (np.abs(P_new) <= eps_up))
    A_new = A_new[rc]
    P_new = P_new[rc]
    rc = np.arange(np.size(A_new))
    
    
    # Initialisation of A_new4, P_new4 and rc4
    rc_new4 = ((A_new < eps_4) & (np.abs(P_new) < eps_4))
    A_new4 = A_new[rc_new4]
    P_new4 = P_new[rc_new4]
    rc4 = rc[rc_new4]

    # Initialisation of A_new3, P_new3 and rc3
    rc_new3 = ((A_new < eps_3) & (np.abs(P_new) < eps_3))
    A_new3 = A_new[rc_new3]
    P_new3 = P_new[rc_new3]
    rc3 = rc[rc_new3]
    
    while ((np.size(rc3) != 1) or (np.size(rc4) != 1)) and (i < 100):
        i += 1
        
        if np.size(A_new4) == 0: # the new interval will be from eps_4 to eps_up
            eps_down = eps_4
            
            eps_4 = eps_3
            A_new4 = A_new3
            P_new4 = P_new3
            rc4 = rc3
            
            interval = eps_up - eps_down
            
            # Recompute eps_3, A_new3, P_new3 and rc_3
            eps_3 = r*interval + eps_down
            rc_3 = ((A_new < eps_3) & (np.abs(P_new) < eps_3))
            A_new3 = A_new[rc_3]
            P_new3 = P_new[rc_3]
            
        else : # New interval will be from eps_down to eps_3
            eps_up = eps_3
            A_new = A_new3
            P_new = P_new3
            rc = rc3
            
            eps_3 = eps_4
            A_new3 = A_new4
            P_new3 = P_new4
            rc3 = rc4
            
            interval = eps_up - eps_down
            
            # Recompute eps_4, A_new4, P_new4 and rc_4
            eps_4 = r**2 * interval + eps_down
            rc_new4 = ((A_new3 < eps_4) & (np.abs(P_new3) < eps_4))
            A_new4 = A_new3[rc_new4]
            P_new4 = P_new3[rc_new4] 
            rc4 = rc3[rc_new4] 
    
    if np.size(A_new4) == 1:
        rc = rc4
    else: 
        rc = rc3
    
    if np.size(rc) != 1: # if the loop as not converge in 100 iterations, NaN values are kept
        H = float('nan')
        B = float('nan')
    else:
        row, col = np.unravel_index(rc[0], (rows, columns))
        H = h[col]
        B = b[row]
    return (H, B)

def compute_Ne(z, H, B):
    
    """Applies Wait's profile to H and B to obtain Ne at altitude z"""
    
    Ne = 1.43e13 * np.exp(-0.15*H)*np.exp((B - 0.15)*(z - H))
    
    return Ne



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Aug 21 10:44:38 2024

@author: pteysseyre
"""

import numpy as np

def flags(amp_NS, amp_EW, previous_nb_points):
    """Flags 1 if the antenna is down, 2 if the transmitter is down"""
    
    flag = 0
    
    length_data = min(len(amp_NS), len(amp_EW)) # this is necessary as there may be one more point in amp_EW than amp_NS, as it is read later
    
    # Antenna down: the number of data points in the last written file stays the same
    if length_data == previous_nb_points:
        flag = 1
    else:
        previous_nb_points = length_data
    
        # Transmitter down: the amplitude is less than 0.3 pT
        
        if length_data > 60:
            amp_NS = amp_NS[-60:]
            amp_EW = amp_EW[-60:]
        else:
            amp_NS = amp_NS[-length_data:]
            amp_EW = amp_EW[-length_data:]
            
        amp = np.sqrt(np.square(amp_NS) + np.square(amp_EW))
        data_last_min = np.median(amp)
    
    if data_last_min < 0.3: # 0.3 pT is the antenna sensibility
        flag = 2
        
    return flag, previous_nb_points, data_last_min
    